/*
 * @(#)CSVAlign.java
 *
 * Summary: align a CSV File into columns.
 *
 * Copyright: (c) 1998-2010 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  2.9 2000-03-27 - refactor using enums, support comments.
 *  3.0 2009-06-15 - lookup table to speed CSVReader
 *  3.1 2009-12-03 - add CSVSort
 *  3.2 2010-02-23 - add hex sort 9x+ option to CSVSort
 *  3.3 2010-11-14 - change default to no comments in input file for CSVTab2Comma.
 *  3.4 2010-12-03 - add CSV2SRS
 *  3.5 2010-12-11 - add CSVReshape
 */
package com.mindprod.csv;

import com.mindprod.common15.STA;
import com.mindprod.hunkio.HunkIO;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import static java.lang.System.err;
import static java.lang.System.out;

/**
 * align a CSV File into columns.
 * <p/>
 * Use: java.exe com.mindprod.CSVAlign somefile.csv
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 3.5 2010-12-11 - add CSVReshape.
 * @since 1998
 */
public final class CSVAlign
    {
    // ------------------------------ CONSTANTS ------------------------------

    /**
     * always use windows line separator on any platform since csv is a windows format file.
     */
    private static final String lineSeparator = "\r\n";

    // ------------------------------ FIELDS ------------------------------

    /**
     * ColumnDescriptors for each column as an ArrayList for pass1
     */
    private ArrayList<ColumnDescriptor> colDescriptorList;

    /**
     * ColumnDescriptors for each column as an array for pass2
     */
    private ColumnDescriptor[] colDescriptors;

    /**
     * where we accumulate an entire aligned line.  Cannot be made local.
     */
    private StringBuilder sb;

    /**
     * The width of an entire aligned line, less the line separator.
     */
    private int alignedLineWidth;

    /**
     * width of longest comment that is not on a line by itself
     */
    private int longestTailComment;

    /**
     * how many pending leading spaces are outstanding.
     * -1 = nothing outstanding.
     * 0 = comma
     * 1 = comma then one space.
     * m.
     */
    private int pending = -1;
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * align a CSV file, constructor. Just create. There are no methods to call.
     *
     * @param file        CSV file to be aligned in columns.
     * @param separator   field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                    tab.
     * @param quote       char to use to enclose fields containing a separator, usually '\"' . Use (char)0 if
     *                    you don't want a quote character.
     * @param commentChar char to use to introduce comments.  Use (char) 0 if none.  Only one character allowed.
     *
     * @throws IOException if problems reading/writing file
     */
    @SuppressWarnings( { "WeakerAccess" } )
    public CSVAlign( File file, char separator, char quote, char commentChar ) throws IOException
    {
    // read file to get max widths
    pass1( file, separator, quote, commentChar );

    // calculate the aggregate width of all the aligned fields.
    calcAlignedLineWidth();

    // read file again this time writing, widening fields to align.
    pass2( file, separator, quote, commentChar );
    }

    // -------------------------- STATIC METHODS --------------------------

    /**
     * prepare an awkward string for output by enclosing it in " and doubling internal "
     *
     * @param s     output field to prepare
     * @param quote char to use to enclose fields containing a separator, usually '\"'
     *
     * @return string enclosed in quotes with internal quotes doubled.
     */
    private static String prepareAwkward( String s, char quote )
    {
    if ( s.indexOf( quote ) < 0 )
        {
        return quote + s + quote;
        }
    else
        {
        StringBuffer sb = new StringBuffer( s.length() + 10 );
        sb.append( quote );
        for ( int i = 0, n = s.length(); i < n; i++ )
            {
            char c = s.charAt( i );
            if ( c == quote )
                {
                sb.append( quote );
                sb.append( quote );
                }
            else
                {
                sb.append( c );
                }
            }// end for
        sb.append( quote );
        return sb.toString();
        }// end else
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * calculate the with of the line after we have aligned it.
     */
    private void calcAlignedLineWidth()
    {
    alignedLineWidth = 0;
    int size = colDescriptorList.size();
    colDescriptors = colDescriptorList.toArray( new ColumnDescriptor[ size ] );
    colDescriptorList = null;
    for ( ColumnDescriptor columnDescriptor : colDescriptors )
        {
        /* one for comma and one for space */
        alignedLineWidth += columnDescriptor.maxWidth + 2;
        }
    }

    /**
     * append a field to the wholeLine left justified
     *
     * @param field       field to add, with awkward encoding in place if any.
     * @param columnWidth width of the column
     */
    private void leftJustify( String field, int columnWidth )
    {
    if ( pending >= 0 )
        {
        sb.append( ',' );
        for ( int j = 0; j < pending; j++ )
            {
            sb.append( ' ' );
            }
        }
    sb.append( field );
    // if there is a subsequent field we have a , plus spaces to pad, plus
    // space.
    pending = columnWidth - field.length() + 1;
    }

    /**
     * pass1 of aligning a CSV file, get the maximum column widths
     *
     * @param file          CSV file to be aligned in columns.
     * @param separatorChar field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                      tab.
     * @param quoteChar     char to use to enclose fields containing a separator, usually '\"'
     * @param commentChar   char to use to introduce comments.  Use (char) 0 if none.  Only one character allowed.
     *
     * @throws IOException if problems reading
     */
    private void pass1( File file,
                        char separatorChar,
                        char quoteChar,
                        char commentChar
    ) throws IOException
    {// first pass read
    final String commentChars = ( commentChar == 0 ) ? "" : String.valueOf( commentChar );
    // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
    // on first pass ignore comments.
    CSVReader csv =
            new CSVReader( new BufferedReader( new FileReader( file ) ), separatorChar, quoteChar, commentChars, false, true, false );
    colDescriptorList = new ArrayList<ColumnDescriptor>( 40 );
    try
        {
        while ( true )
            {
            final String[] fields = csv.getAllFieldsInLine();

            boolean wasComment = csv.wasComment();

            for ( int colIndex = 0; colIndex < fields.length; colIndex++ )
                {
                if ( wasComment && colIndex == fields.length - 1 )
                    {
                    // single line comments we ignore. We presume they will not affect average width.
                    if ( colIndex != 0 )
                        {
                        final int length = fields[ colIndex ].length();
                        if ( length > longestTailComment )
                            {
                            longestTailComment = length;
                            }
                        }
                    }
                else
                    {
                    if ( colIndex >= colDescriptorList.size() )
                        {
                        colDescriptorList.add( new ColumnDescriptor() );
                        }
                    ColumnDescriptor col = colDescriptorList.get( colIndex );
                    String field = fields[ colIndex ];
                    int width = field.length();
                    final boolean loneEmptyField = fields.length == ( wasComment ? 2 : 1 ) && fields[ 0 ].trim().length() == 0;
                    // does this field need surrounding quotes?
                    boolean isAwkward = col.isAwkward
                                        || loneEmptyField
                                        || field.indexOf( separatorChar ) >= 0
                                        || field.indexOf( quoteChar ) >= 0
                                        || field.indexOf( commentChar ) >= 0;
                    if ( isAwkward )
                        {
                        // extra col for lead/trail quote.
                        width += 2;
                        // extra col for each doubled quote
                        for ( int i = 0; i < field.length(); i++ )
                            {
                            if ( field.charAt( i ) == quoteChar )
                                {
                                width++;
                                }
                            }
                        }
                    if ( !col.isAwkward && isAwkward )
                        {
                        col.isAwkward = true;
                        // earlier non-awkwards will be displayed in quotes.
                        col.maxWidth += 2;

                        out.println( "    Note : col " + colIndex + " quoted because of line " + csv.getLine() + " : " + field );
                        }
                    if ( width > col.maxWidth )
                        {
                        col.maxWidth = width;
                        }
                    if ( !STA.isLegal( field, "0123456789.+-" ) )
                        {
                        col.isNumeric = false;
                        }
                    }
                }
            }
        }
    catch ( EOFException e )
        {
        // expected
        }
    csv.close();
    }

    /**
     * pass2 of aligning a CSV file, expand fields to the max width
     *
     * @param file          CSV file to be aligned in columns.
     * @param separatorChar field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                      tab.
     * @param quoteChar     char to use to enclose fields containing a separator, usually '\"'
     * @param commentChar   char to use to introduce comments.  Use (char) 0 if none.  Only one character allowed.
     *
     * @throws IOException if problems reading/writing file
     */
    private void pass2( File file,
                        char separatorChar,
                        char quoteChar,
                        char commentChar ) throws IOException
    {
    final String commentChars = ( commentChar == 0 ) ? "" : String.valueOf( commentChar );
    // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
    // on second pass, consider comments.
    CSVReader csv =
            new CSVReader( new BufferedReader( new FileReader( file ) ), separatorChar, quoteChar, commentChars, false, true, false );
    File tempFile = HunkIO.createTempFile( "temp", ".tmp", file );
    // we don't use CSVWriter to write the output.
    BufferedWriter out = new BufferedWriter( new FileWriter( tempFile ) );

    try
        {
        while ( true )
            {
            final String[] fields = csv.getAllFieldsInLine();
            final boolean wasComment = csv.wasComment();

            sb = new StringBuilder( alignedLineWidth + longestTailComment + 3 /* space # space */ );
            pending = -1;
            // may have short line.
            for ( int colIndex = 0; colIndex < fields.length; colIndex++ )
                {
                if ( wasComment && colIndex == fields.length - 1 )
                    {
                    if ( colIndex == 0 )
                        {
                        // comment on line by itself, on left
                        sb.append( commentChar );
                        sb.append( ' ' );
                        sb.append( fields[ colIndex ] );
                        }
                    else
                        {
                        // tail comment, put out past end of all fields, all aligned.
                        while ( sb.length() < alignedLineWidth )
                            {
                            sb.append( ' ' );
                            }
                        sb.append( ' ' );
                        sb.append( commentChar );
                        sb.append( ' ' );
                        sb.append( fields[ colIndex ] );
                        }
                    }
                else
                    {
                    final ColumnDescriptor col = colDescriptors[ colIndex ];
                    String field = fields[ colIndex ];
                    if ( col.isNumeric )
                        {
                        // right justify, can't be awkward
                        // spaces, field, comma, space
                        rightJustify( field, col.maxWidth );
                        }
                    else
                        {
                        // left justify
                        // field, comma, space, spaces
                        if ( col.isAwkward )
                            {
                            // also handles loneEmptyField
                            field = prepareAwkward( field, quoteChar );
                            }
                        leftJustify( field, col.maxWidth );
                        }
                    }
                }// end for
            out.write( STA.trimTrailing( sb.toString() ) );
            out.write( lineSeparator );
            }// end while
        }// end try
    catch ( EOFException e )
        {
        csv.close();
        // swap tempFile output and input now that output is safely created.
        out.close();
        file.delete();
        tempFile.renameTo( file );
        }
    }

    /**
     * append a field to the wholeLine right justified
     *
     * @param field       field to add, with awkward encoding in place if any.
     * @param columnWidth width of the column
     */
    private void rightJustify( String field, int columnWidth )
    {
    if ( pending >= 0 )
        {
        sb.append( ',' );
        for ( int j = 0; j < pending; j++ )
            {
            sb.append( ' ' );
            }
        }
    int spaces = columnWidth - field.length();
    for ( int j = 0; j < spaces; j++ )
        {
        sb.append( ' ' );
        }
    sb.append( field );
    pending = 1;/* comma plus space, if there is a subsequent field */
    }

    // --------------------------- main() method ---------------------------

    /**
     * Simple command line interface to AlignCSVFile. Aligns one file whose name is on the command line. Must have
     * extension .csv <br> Use java com.mindprod.CSVAlign somefile.csv
     *
     * @param args name of csv file to align in columns.
     */
    public static void main( String[] args )
    {
    if ( args.length != 1 )
        {
        throw new IllegalArgumentException(
                "CSVAlign needs a single filename.csv on the command line." );
        }
    String filename = args[ 0 ];
    if ( !filename.endsWith( ".csv" ) )
        {
        throw new IllegalArgumentException(
                "Bad extension.\nCSVAlign needs a single filename.csv on the command line." );
        }
    try
        {
        // file , separatorChar, quoteChar, commentChar
        new CSVAlign( new File( filename ), ',', '\"', '#' );
        }
    catch ( IOException e )
        {
        err.println( "CSVAlign failed to align " + filename );
        err.println( e.getMessage() );
        e.printStackTrace();
        }
    }// end main
    }

/**
 * Facts about a single column in the CSV file, used internally by CSVAlign
 * <p/>
 * created with Intellij Idea
 *
 * @author Roedy Green, Canadian Mind Products
 */
final class ColumnDescriptor
    {
    // ------------------------------ FIELDS ------------------------------

    /**
     * true if one or more fields in this column require surrounding quotes. On output, all fields in an awkward column
     * will get surrounding quotes. If a field contains , or quote it needs the surrounding quotes.
     */
    public boolean isAwkward = false;

    /**
     * if true all fields contain only digits dot, plus and minus. Assume column is numeric until proven otherwise by
     * finding a non-numeric char.
     */
    public boolean isNumeric = true;

    /**
     * widest field in the column. Not including comma. If a column is "awkward" (needs surrounding quotes), then the
     * length includes the quotes and any internal quoting.
     */
    public int maxWidth = 0;
    }
