/*
 * @(#)CSVDump.java
 *
 * Summary: display a csv file with fields separated with |.
 *
 * Copyright: (c) 1998-2010 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  2.9 2000-03-27 - refactor using enums, support comments.
 *  3.0 2009-06-15 - lookup table to speed CSVReader
 *  3.1 2009-12-03 - add CSVSort
 *  3.2 2010-02-23 - add hex sort 9x+ option to CSVSort
 *  3.3 2010-11-14 - change default to no comments in input file for CSVTab2Comma.
 *  3.4 2010-12-03 - add CSV2SRS
 *  3.5 2010-12-11 - add CSVReshape
 */
package com.mindprod.csv;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import static java.lang.System.err;
import static java.lang.System.out;

/**
 * display a csv file with fields separated with |.
 * <p/>
 * Use: java.exe com.mindprod.CSVDump somefile.csv
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 3.5 2010-12-11 - add CSVReshape
 * @since 1998
 */
public final class CSVDump
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * Dump CSV file.
     *
     * @param file         CSV file to be dumped
     * @param separator    field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                     tab.
     * @param quote        char to use to enclose fields containing a separator, usually '\"'. Use (char)0 if
     *                     you don't want a quote character.
     * @param commentChars characters that mark the start of a comment, usually "#", but can be multiple chars.
     *                     Note this is a "string" not a 'char'.
     *
     * @throws IOException if problems reading file
     */
    public CSVDump( File file, char separator, char quote, String commentChars ) throws IOException
    {
    // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
    CSVReader in = new CSVReader( new BufferedReader( new FileReader( file ) ), separator, quote, commentChars, false, true, true );

    try
        {
        while ( true )
            {
            // read one line:
            final String[] fields = in.getAllFieldsInLine();
            for ( String field : fields )
                {
                out.print( "| " );
                out.print( field );  // comment treated like any other field.
                out.print( ' ' );
                }
            if ( in.wasComment() )
                {
                out.println( "<<" );
                }
            else
                {
                out.println( "|" );
                }
            }
        }
    catch ( EOFException e )
        {
        in.close();
        }
    }

    // --------------------------- main() method ---------------------------

    /**
     * Simple command line interface to CSVPack. Packs one csv file whose name is on the command line. Must have
     * extension .csv <br> Use java com.mindprod.CSVPack somefile.csv
     *
     * @param args name of csv file to remove excess quotes and space
     */
    public static void main
    ( String[] args )
    {
    if ( args.length != 1 )
        {
        throw new IllegalArgumentException(
                "CSVDump needs a single filename.csv on the command line." );
        }
    String filename = args[ 0 ];
    if ( !filename.endsWith( ".csv" ) )
        {
        throw new IllegalArgumentException(
                "Bad Extension\nCSVDump needs a single filename.csv on the command line." );
        }
    try
        {
        new CSVDump( new File( filename ), ',', '\"', "#" );
        }
    catch ( IOException e )
        {
        err.println( "CSVDump failed " + filename );
        err.println( e.getMessage() );
        e.printStackTrace();
        }
    }// end main
    }
