/*
 * @(#)CSVPack.java
 *
 * Summary: pack a CSV File as densely as possible, removing excess space and commas.
 *
 * Copyright: (c) 1998-2010 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  2.9 2000-03-27 - refactor using enums, support comments.
 *  3.0 2009-06-15 - lookup table to speed CSVReader
 *  3.1 2009-12-03 - add CSVSort
 *  3.2 2010-02-23 - add hex sort 9x+ option to CSVSort
 *  3.3 2010-11-14 - change default to no comments in input file for CSVTab2Comma.
 *  3.4 2010-12-03 - add CSV2SRS
 *  3.5 2010-12-11 - add CSVReshape
 */
package com.mindprod.csv;

import com.mindprod.hunkio.HunkIO;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import static java.lang.System.err;

/**
 * pack a CSV File as densely as possible, removing excess space and commas.
 * <p/>
 * Use: java.exe com.mindprod.CSVPack somefile.csv
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 3.5 2010-12-11 - add CSVReshape
 * @since 1998
 */
public final class CSVPack
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * pack a CSV file, constructor. Just create. There are no methods to call.
     *
     * @param file          CSV file to be packed to remove excess space and quotes.
     * @param separatorChar field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                      tab.
     * @param quoteChar     char to use to enclose fields containing a separator, usually '\"'. Use (char)0 if
     *                      you don't want a quote character.
     * @param commentChar   char to use to introduce comments.  Use (char) 0 if none.  Only one character allowed.
     *
     * @throws IOException if problems reading/writing file
     */
    @SuppressWarnings( { "WeakerAccess" } )
    public CSVPack( File file, char separatorChar, char quoteChar, char commentChar ) throws IOException
    {
    final String commentChars = ( commentChar == 0 ) ? "" : String.valueOf( commentChar );

    // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
    CSVReader in = new CSVReader( new BufferedReader( new FileReader( file ) ), separatorChar, quoteChar, commentChars, false, true, true );

    File tempFile = HunkIO.createTempFile( "temp", ".tmp", file );

    // writer, quoteLevel, separatorChar, quoteChar, commentChar, trim
    CSVWriter out = new CSVWriter( new PrintWriter( new BufferedWriter( new FileWriter( tempFile ) ) ), 0 /* minimal  */, separatorChar, quoteChar, commentChar, true );

    try
        {
        while ( true )
            {
            final String s = in.get();
            // null means end of line

            if ( in.wasComment() )
                {
                // was a comment
                out.nl( s );
                in.skipToNextLine();
                }
            else
                {
                // null will start a new line.
                out.put( s );
                }
            }
        }
    catch ( EOFException e )
        {
        in.close();
        out.close();
        file.delete();
        tempFile.renameTo( file );
        }
    }

    // --------------------------- main() method ---------------------------

    /**
     * Simple command line interface to CSVPack. Packs one csv file whose name is on the command line. Must have
     * extension .csv <br> Use java com.mindprod.CSVPack somefile.csv
     *
     * @param args name of csv file to remove excess quotes and space
     */
    public static void main( String[] args )
    {
    if ( args.length != 1 )
        {
        throw new IllegalArgumentException(
                "CSVPack needs a single filename.csv on the command line." );
        }
    String filename = args[ 0 ];
    if ( !filename.endsWith( ".csv" ) )
        {
        throw new IllegalArgumentException(
                "Bad Extension\nCSVPack needs a single filename.csv on the command line." );
        }
    try
        {
        // file,  separatorChar,  quoteChar,  commentChar
        new CSVPack( new File( filename ), ',', '\"', '#' );
        }
    catch ( IOException e )
        {
        err.println( "CSVPack failed to pack " + filename );
        err.println( e.getMessage() );
        e.printStackTrace();
        }
    }// end main
    }
