/*
 * @(#)CSVReader.java
 *
 * Summary: Read CSV (Comma Separated Value) files.
 *
 * Copyright: (c) 2002-2010 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  3.0 2009-06-15 - lookup table to speed CSVReader
 *  3.1 2009-12-03 - add CSVSort
 *  3.2 2010-02-23 - add hex sort 9x+ option to CSVSort
 *  3.3 2010-11-14 - change default to no comments in input file for CSVTab2Comma.
 *  3.4 2010-12-03 - add CSV2SRS
 *  3.5 2010-12-11 - add CSVReshape
 *  3.6 2010-12-14 - add LinesToCSV
 *  3.7 2010-12-17 - add CSVDeDup
 */

package com.mindprod.csv;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;

import static com.mindprod.csv.CSVCharCategory.*;
import static com.mindprod.csv.CSVReadState.*;
import static java.lang.System.out;

/*
TODO:
>I see the tests include embedded newlines, "\n", but not embedded
>carriage returns, "\r". On my platform, a descendant of BSD unix called
>Darwin, the result of `ant test` shows no errors.

That was intentional.
I figured it would be easier to deal with field results in canonical \n form, easy to split.
I need to check this out, and document the feature, and make sure if such fields
are written out CSVWriter honours the line separator convention.
*/

/**
 * Read CSV (Comma Separated Value) files.
 * <p/>
 * This format is used my Microsoft Word and Excel. Fields are separated by
 * commas, and enclosed in quotes if they contain commas or quotes. Embedded quotes are doubled. Embedded spaces do not
 * normally require surrounding quotes. The last field on the line is not followed by a comma. Null fields are
 * represented by two commas in a row. We optionally trim leading and trailing spaces on fields, even inside quotes.
 * File must normally end with a single CrLf, other wise you will get a null when trying to read a field on older JVMs.
 * <p/>
 * Must be combined with your own code.
 * <p/>
 * There is another CSVReader at: at http://ostermiller.org/utils/ExcelCSV.html
 * If this CSVReader is not suitable for you, try that one. <br> There is one written in C# at
 * http://www.csvreader.com/
 * <p/>
 * Future ideas:
 * <p/>
 * 1. allow \ to be used for quoting characters.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 3.7 2010-12-17 - add CSVDeDup
 * @since 2002
 */
public final class CSVReader
    {
    // ------------------------------ CONSTANTS ------------------------------

    /**
     * true if want debugging output and debugging test harness code
     */
    private static final boolean DEBUGGING = false;

    /**
     * e.g. \n \r\n or \r, whatever system uses to separate lines in a text file. Only used inside multiline fields. The
     * file itself should use Windows format \r \n, though \n by itself will also work.
     */
    private static final String lineSeparator = System.getProperty( "line.separator" );

    // ------------------------------ FIELDS ------------------------------

    /**
     * Reader source of the CSV fields to be read.
     */
    private BufferedReader r;

    /**
     * table to for quick lookup of char category for chars 0..255
     */
    private CSVCharCategory[] lookup;

    /**
     * finite state automaton for parsing characters on the line
     */
    private CSVReadState state = SEEKING_FIELD;

    /**
     * what char start off a comment.
     */
    private final String commentChars;

    /**
     * state of the parser's finite state automaton. The line we are parsing. null means none read yet. Line contains
     * unprocessed chars. Processed ones are removed.
     */
    private String line = null;

    /**
     * true if reader should allow quoted fields to span more than one line. Microsoft Excel sometimes generates files
     * like this.
     */
    private final boolean allowMultiLineFields;

    /**
     * true means client does not see the comments.  It as if they were not there.
     */
    private final boolean hideComments;

    /**
     * true if quoted fields are trimmed of lead/trail blanks.  Unquoted strings are always trimmed.
     */
    private final boolean trimQuoted;

    /**
     * true if can use fast table lookup to categorise chars, later set true when table built.
     */
    private boolean useLookup = false;

    /**
     * true if last field returned via get was a comment.
     */
    private boolean wasComment;

    /**
     * quote character, usually '\"' '\'' for SOL used to enclose fields containing a separator character.
     */
    private final char quoteChar;

    /**
     * field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for tab.
     */
    private final char separatorChar;

    /**
     * how many fields have been processed on this line so far, not counting comments or EOL
     */
    private int fieldCount = 0;

    /**
     * How many lines we have read so far. Used in error messages.
     */
    private int lineCount = 0;

    /**
     * points to next character to process in the current line.
     */
    private int offset;
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * convenience Constructor, default to comma separator, " for quote, no multiline fields, with trimming.
     *
     * @param r input Reader source of CSV Fields to read.  Should be buffered.
     */
    public CSVReader( Reader r )
    {
    // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
    this( r, ',', '\"', "#", true, true, false );
    }

    /**
     * Constructor
     *
     * @param r                    input Reader source of CSV Fields to read.  Should be a BufferedReader.
     * @param separatorChar        field separator character, usually ',' in North America, ';' in Europe and sometimes
     *                             '\t' for tab. Note this is a 'char' not a "string".
     * @param quoteChar            char to use to enclose fields containing a separator, usually '\"' . Use (char)0 if
     *                             you don't want a quote character, or any other char that will not appear in the file.
     *                             Note this is a 'char' not a "string".
     * @param commentChars         characters that mark the start of a comment, usually "#", but can be multiple chars.
     *                             Note this is a "string" not a 'char'.
     * @param hideComments         true if clients sees none of the comments.  false if client processes the comments.
     * @param trimQuoted           true if quoted fields are trimmed of lead/trail blanks.  Unquoted strings and comments are always trimmed.
     * @param allowMultiLineFields true if reader should allow quoted fields to span more than one line. Microsoft Excel
     */
    public CSVReader( final Reader r,
                      final char separatorChar,
                      final char quoteChar,
                      final String commentChars,
                      final boolean hideComments,
                      final boolean trimQuoted,
                      final boolean allowMultiLineFields
    )
    {
    /* convert Reader to BufferedReader if necessary */
    this.r = ( r instanceof BufferedReader ) ? ( BufferedReader ) r : new BufferedReader( r );    /* default buffer size is 8K */
    this.separatorChar = separatorChar;
    this.quoteChar = quoteChar;
    this.commentChars = commentChars;
    this.hideComments = hideComments;
    this.trimQuoted = trimQuoted;
    this.allowMultiLineFields = allowMultiLineFields;
    buildLookup();
    }

    /**
     * Close the Reader.
     *
     * @throws IOException if problems closing
     */
    public void close() throws IOException
    {
    if ( r != null )
        {
        r.close();
        r = null;
        }
    }

    /**
     * Read one field from the CSV file. You can also use methods like getInt and getDouble to parse the String for you.
     * You can use getAllFieldsInLine to read the entire line including the EOL.
     *
     * @return String value, even if the field is numeric. Surrounded and embedded double quotes are stripped. possibly
     *         "". null means end of line.  Normally you use skiptoNextLine to start the next line rather then using get
     *         to read the eol.  Might also be a comment, with lead # stripped.
     *         If field was a comment, it is returned with lead # stripped. Check wasComment to see if it was a comment
     *         or a data field.
     * @throws EOFException at end of file after all the fields have been read.
     * @throws IOException  Some problem reading the file, possibly malformed data.
     * @noinspection UnusedLabel
     */
    public String get() throws IOException
    {
    // we need a StringBuilder since we analyse double " and handle multiline fields.
    // We can't an simply track start and end of string.
    StringBuilder field = new StringBuilder( allowMultiLineFields ? 512 : 64 );

    /* don't need to maintain state between fields, just offset where to continue processing line. */
    this.wasComment = false;
    lineLoop:
    while ( true )
        {
        getLineIfNeeded();

        charLoop:
        /* loop for each char in the line to find a field */
        /* guaranteed to leave early by hitting EOL */

        // pick up from where we left off getting last field.
        for ( int i = offset; i < line.length(); i++ )
            {
            char c = line.charAt( i );
            CSVCharCategory category = categorise( c );
            if ( DEBUGGING )
                {
                // for debugging
                out.println( "char:"
                             + c
                             + " state:"
                             + state
                             + " field:"
                             + field.length() );
                }
            // nested switch state:char category
            switch ( state )
                {
                case AFTER_END_QUOTE:
                {
                /*
                * In situation like this "xxx" which may turn out to be
                * xxx""xxx" or "xxx", We find out here.
                */
                switch ( category )
                    {
                    case COMMENT_START:
                        // either skip over comment or process it later
                        offset = hideComments ? line.length() : i;
                        // handle pending field
                        state = SEEKING_FIELD;
                        fieldCount++;
                        return trimQuoted ? field.toString().trim() : field.toString();

                    case ORDINARY:
                        throw new IOException(
                                "Malformed CSV stream, missing ["
                                + separatorChar
                                + "] near offset "
                                + i
                                + " after field"
                                + fieldCount
                                + " on line "
                                + lineCount );

                    case QUOTE:
                        /* was a double quotechar, e.g. a literal " */
                        field.append( c );
                        state = IN_QUOTED;
                        break;

                    case SEPARATOR:
                        /* we are done with field. */
                        offset = i + 1;
                        state = SEEKING_START;
                        fieldCount++;
                        return trimQuoted ? field.toString().trim() : field.toString();

                    case WHITESPACE:
                        /* ignore trailing spaces up to separatorChar */
                        state = SKIPPING_TAIL;
                        break;
                    }
                break;
                }

                case IN_PLAIN:
                {
                /* in middle of ordinary field */
                switch ( category )
                    {
                    case COMMENT_START:
                        // either skip over comment, or handle it later
                        offset = hideComments ? line.length() : i;
                        // handle pending field
                        state = SEEKING_FIELD;
                        fieldCount++;
                        return field.toString().trim();

                    case ORDINARY:
                        field.append( c );
                        break;

                    case QUOTE:
                        throw new IOException(
                                "Malformed CSV stream. Missing ["
                                + quoteChar
                                + "] near offset "
                                + i
                                + " at start of field "
                                + ( fieldCount + 1 )
                                + " on line "
                                + lineCount );

                    case SEPARATOR:
                        /* done */
                        offset = i + 1;
                        state = SEEKING_START;
                        fieldCount++;
                        return field.toString().trim();

                    case WHITESPACE:
                        field.append( ' ' );
                        break;
                    }
                break;
                }

                case IN_QUOTED:
                {
                /* in middle of field surrounded in quotes */
                switch ( category )
                    {
                    case COMMENT_START:  // inside quotes only " is a special character.
                    case ORDINARY:
                    case SEPARATOR:
                    case WHITESPACE:
                        field.append( c );
                        break;

                    case QUOTE:
                        state = AFTER_END_QUOTE;
                        break;
                    }
                break;
                }

                case SEEKING_FIELD:
                {
                /* in blanks before first field */
                switch ( category )
                    {
                    case COMMENT_START:
                        if ( hideComments )
                            {
                            if ( fieldCount > 0 )
                                { // bypass comment
                                offset = line.length(); /* carry on at eol */
                                state = SEEKING_FIELD;
                                break charLoop;
                                }
                            else
                                {
                                // comment on by itself. ignore the whole thing, including the EOL.
                                line = null;
                                state = SEEKING_FIELD;
                                continue lineLoop;
                                }
                            }
                        else
                            {
                            // entire rest of line is a comment.
                            offset = line.length();/* carry on at eol */
                            state = SEEKING_FIELD;
                            // don't increment fieldCount, this is a comment field.
                            wasComment = true;
                            // strip off lead #
                            return line.substring( i + 1 ).trim();
                            }

                    case QUOTE:
                        state = IN_QUOTED;
                        break;

                    case SEPARATOR:
                        /* end of empty field */
                        offset = i + 1;
                        state = SEEKING_START;
                        fieldCount++;
                        return "";

                    case ORDINARY:
                        field.append( c );
                        state = IN_PLAIN;
                        break;

                    case WHITESPACE:
                        /* ignore */
                        break;
                    }
                break;
                }

                case SEEKING_START:
                {
                /* in blanks before field */
                switch ( category )
                    {
                    case COMMENT_START:
                        // either bypass comment or arrange to deal with it later
                        offset = hideComments ? line.length() : i;
                        // handle pending empty field
                        state = SEEKING_FIELD;
                        fieldCount++;
                        return "";

                    case QUOTE:
                        state = IN_QUOTED;
                        break;

                    case SEPARATOR:
                        /* end of empty field */
                        offset = i + 1;
                        state = SEEKING_START;
                        fieldCount++;
                        return "";

                    case ORDINARY:
                        field.append( c );
                        state = IN_PLAIN;
                        break;

                    case WHITESPACE:
                        /* ignore */
                        break;
                    }
                break;
                }
                case SKIPPING_TAIL:
                {
                /* in spaces after quoted field, seeking separatorChar */

                switch ( category )
                    {
                    case COMMENT_START:
                        // handle pending field, deal with comment later.
                        offset = i;
                        state = SEEKING_FIELD;
                        fieldCount++;
                        return trimQuoted ? field.toString().trim() : field.toString();

                    case ORDINARY:
                    case QUOTE:
                        throw new IOException(
                                "Malformed CSV stream, missing ["
                                + separatorChar
                                + "] near offset "
                                + i
                                + "after field "
                                + fieldCount
                                + " on line "
                                + lineCount );

                    case SEPARATOR:
                        offset = i + 1;
                        state = SEEKING_START;
                        fieldCount++;
                        return trimQuoted ? field.toString().trim() : field.toString();

                    case WHITESPACE:
                        /* ignore trailing spaces up to separatorChar */
                        break;
                    }
                break;
                }
                }// end switch(state)
            }
        // end charLoop over remaining chars in line.

        if ( DEBUGGING )
            {
            // for debugging
            out.println( "EOL state:"
                         + state
                         + " field:"
                         + field.length() );
            }
        // if not found a field yet, handle the end of line.
        switch ( state )
            {
            case AFTER_END_QUOTE:
                /*
                 * In situation like this "xxx" which may turn out to be
                 * xxx""xxx" or "xxx", We find out here.
                */
                offset = line.length();/* carry on at eol */
                state = SEEKING_FIELD;
                fieldCount++;
                return trimQuoted ? field.toString().trim() : field.toString();

            case SEEKING_FIELD:
                /* in blanks prior to start of a field  */
                /* null to mark end of line */
                // mark line as done, we need a new line of characters.
                // carry on at start of next line.
                line = null;
                state = SEEKING_FIELD;
                // return null for EOL
                return null;

            case SEEKING_START:
                /* in blanks after , */
                /* null to mark end of line */
                offset = line.length();/* carry on at eol */
                // handle pending empty field
                state = SEEKING_FIELD;
                fieldCount++;
                return "";

            case IN_PLAIN:
                /* in middle of ordinary field */
                offset = line.length(); /* push EOL back */
                state = SEEKING_FIELD;
                fieldCount++;
                return field.toString().trim();

            case IN_QUOTED:
                /* in middle of field surrounded in quotes */
                if ( allowMultiLineFields )
                    {
                    field.append( lineSeparator );
                    // we are done with that line, but not with
                    // the
                    // field.
                    // We don't want to return a null
                    // to mark the end of the line., but we do want another line to process.
                    line = null;
                    // will read next line and seek the end of
                    // the quoted field with state = IN_QUOTED.
                    break;
                    }
                else
                    {
                    // no multiline fields allowed
                    throw new IOException(
                            "Malformed CSV stream. Missing ["
                            + quoteChar
                            + "] near offset "
                            + ( line.length() - 1 )
                            + " after field "
                            + fieldCount
                            + " on line "
                            + lineCount );
                    }

            case SKIPPING_TAIL:
                offset = line.length();/* carry on at eol */
                state = SEEKING_FIELD;
                fieldCount++;
                return trimQuoted ? field.toString().trim() : field.toString();
            } // end switch
        }
    // end lineLoop
    }// end get

    /**
     * Get all fields in the line. This reads only one line, not the whole file.
     * Skips to next line as a side effect, so don't need skipToNextLine.
     * Can find out if last field was a comment with wasComment();
     *
     * @return Array of strings, one for each field. Possibly empty, but never null.
     * @throws EOFException if run off the end of the file.
     * @throws IOException  if some problem reading the file.
     */
    public String[] getAllFieldsInLine() throws IOException
    {
    ArrayList<String> al = new ArrayList<String>( 100 );
    boolean lineHadComment = false;
    do
        {
        String field = get();
        if ( wasComment() )
            {
            lineHadComment = true;
            }
        if ( field == null )
            {
            break;
            }
        al.add( field );
        }
    while ( true );

    if ( lineHadComment )
        {
        wasComment = true;
        }
    return al.toArray( new String[ al.size() ] );
    }

    /**
     * Read one Boolean field from the CSV file, e.g. true, yes, 1 or false, no, 0.
     *
     * @return boolean, empty field returns false, as does end of line.
     * @throws EOFException           at end of file after all the fields have been read.
     * @throws IOException            Some problem reading the file, possibly malformed data.
     * @throws NumberFormatException, if field does not contain a well-formed int.
     * @noinspection UnusedDeclaration
     */
    public boolean getBoolean() throws IOException
    {
    String s = get();
    // end of line returns 0
    if ( s == null || s.length() == 0 )
        {
        return false;
        }
    if ( s.equalsIgnoreCase( "true" ) || s.equalsIgnoreCase( "yes" ) || s.equals( "1" ) )
        {
        return true;
        }
    if ( s.equalsIgnoreCase( "false" ) || s.equalsIgnoreCase( "no" ) || s.equals( "0" ) )
        {
        return false;
        }
    throw new IOException(
            "Malformed boolean near offset "
            + ( line.length() - 1 )
            + " after field "
            + fieldCount
            + " on line "
            + lineCount );
    }

    /**
     * Read one double field from the CSV file.
     *
     * @return double value, empty field returns 0, as does end of line.
     * @throws EOFException           at end of file after all the fields have been read.
     * @throws IOException            Some problem reading the file, possibly malformed data.
     * @throws NumberFormatException, if field does not contain a well-formed double.
     * @noinspection UnusedDeclaration
     */
    public double getDouble() throws IOException, NumberFormatException
    {
    String s = get();
    if ( s == null || s.length() == 0 )
        {
        return 0;
        }

    return Double.parseDouble( s );
    }

    /**
     * Read one float field from the CSV file.
     *
     * @return float value, empty field returns 0, as does end of line.
     * @throws EOFException           at end of file after all the fields have been read.
     * @throws IOException            Some problem reading the file, possibly malformed data.
     * @throws NumberFormatException, if field does not contain a well-formed float.
     * @noinspection UnusedDeclaration
     */
    public float getFloat() throws IOException, NumberFormatException
    {
    String s = get();
    if ( s == null || s.length() == 0 )
        {
        return 0;
        }
    return Float.parseFloat( s );
    }

    /**
     * Read one integer field from the CSV file
     *
     * @return int value, empty field returns 0, as does end of line.
     * @throws EOFException           at end of file after all the fields have been read.
     * @throws IOException            Some problem reading the file, possibly malformed data.
     * @throws NumberFormatException, if field does not contain a well-formed int.
     * @noinspection UnusedDeclaration
     */
    public int getInt() throws IOException, NumberFormatException
    {
    String s = get();
    // end of line returns 0
    if ( s == null || s.length() == 0 )
        {
        return 0;
        }
    return Integer.parseInt( s );
    }

    /**
     * How many lines have been processed so far.
     */
    public int getLine()
    {
    return lineCount;
    }

    /**
     * Read one long field from the CSV file
     *
     * @return long value, empty field returns 0, as does end of line.
     * @throws EOFException           at end of file after all the fields have been read.
     * @throws IOException            Some problem reading the file, possibly malformed data.
     * @throws NumberFormatException, if field does not contain a well-formed long.
     * @noinspection UnusedDeclaration
     */
    public long getLong() throws IOException, NumberFormatException
    {
    String s = get();
    if ( s == null || s.length() == 0 )
        {
        return 0;
        }
    return Long.parseLong( s );
    }

    /**
     * Skip over fields you don't want to process.
     *
     * @param fields How many field you want to bypass reading. The newline counts as one field.
     *
     * @throws EOFException at end of file after all the fields have been read.
     * @throws IOException  Some problem reading the file, possibly malformed data.
     * @noinspection UnusedDeclaration
     */
    public void skip( int fields ) throws IOException
    {
    if ( fields <= 0 )
        {
        return;
        }
    for ( int i = 0; i < fields; i++ )
        {
        // throw results away
        get();
        }
    }

    /**
     * Skip over remaining fields on this line you don't want to process.
     *
     * @throws EOFException at end of file after all the fields have been read.
     * @throws IOException  Some problem reading the file, possibly malformed data.
     */
    public void skipToNextLine() throws IOException
    {
    if ( line == null )
        {
        getLineIfNeeded();
        }
    line = null;
    }

    /**
     * Was the last field returned via get a comment?  Also works after getAllFieldsInLine to tell if there was a comment at the end of that line.
     *
     * @return true if last field returned via get was a comment.
     */
    public boolean wasComment()
    {
    return this.wasComment;
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * build table to for quick lookup of char category.
     */
    void buildLookup()
    {
    lookup = new CSVCharCategory[ 256 ];
    for ( int c = 0; c < 256; c++ )
        {
        lookup[ c ] = categorise( ( char ) c );
        }
    useLookup = true; // table now safe to use
    }

    /**
     * categorise a character for the finite state machine.
     *
     * @param c the character to categorise
     *
     * @return integer representing the character's category.
     */
    CSVCharCategory categorise( char c )
    {
    if ( useLookup && c < 256 )
        {
        return lookup[ c ];
        }
    switch ( c )
        {
        case ' ':
        case '\r':
        case '\n':
        case 0xff:
            return WHITESPACE;

        default:
            if ( c == separatorChar )
                {
                return SEPARATOR;
                }
            else if ( c == quoteChar )
                {
                return QUOTE;
                }
            if ( commentChars.indexOf( c ) >= 0 )
                {
                return COMMENT_START;
                }
            /* do our tests in crafted order, hoping for an early return */
            else if ( '!' <= c && c <= '~' )/* includes A-Z a-z 0-9 common punctuation */
                {
                return ORDINARY;
                }
            else if ( 0x00 <= c && c <= 0x20 || Character.isWhitespace( c ) )
                {
                return WHITESPACE;
                }
            else
                {
                return ORDINARY;
                }
        }
    }

    /**
     * Make sure a line is available for parsing. Does nothing if there already is one.
     *
     * @throws EOFException if hit the end of the file.
     */
    private void getLineIfNeeded() throws IOException
    {
    if ( line == null )
        {
        if ( r == null )
            {
            throw new IllegalArgumentException(
                    "attempt to use a closed CSVReader" );
            }
        line = r.readLine();/* this strips platform specific line ending */
        if ( line == null )/*
                                 * null means EOF, yet another inconsistent Java
                                 * convention.
                                 */
            {
            throw new EOFException();
            }
        else
            {
            state = SEEKING_FIELD;
            offset = 0;
            fieldCount = 0;
            lineCount++;
            }
        }
    }

    // --------------------------- main() method ---------------------------

    /**
     * Test driver
     *
     * @param args not used
     *
     * @noinspection InfiniteLoopStatement
     */
    public static void main( String[] args )
    {
    if ( DEBUGGING )
        {
        try
            {
            // read test file
            // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
            CSVReader csv =
                    new CSVReader( new BufferedReader( new FileReader( "test.csv" ) ),
                            ',',
                            '\"',
                            "#",
                            false /* hide comments */,
                            true /*  trim */,
                            true /* multiline */
                    );
            try
                {
                while ( true )
                    {
                    final String s = csv.get();
                    if ( csv.wasComment() )
                        {
                        out.println( "   --> <" + s + ">" );
                        }
                    else
                        {
                        out.println( "--> [" + s + "]" );
                        }
                    }
                }
            catch ( EOFException e )
                {
                // normal termination.
                }
            csv.close();
            }
        catch ( IOException e )
            {
            e.printStackTrace();
            out.println( e.getMessage() );
            }
        }// end if
    }// end main
    }// end CSVReader class.
