/*
 * @(#)CSVReshape.java
 *
 * Summary: reshape a CSV file reordering, duplicating or removing fields.
 *
 * Copyright: (c) 2010 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  3.5 2010-12-11 - add CSVReshape initial version
 */
package com.mindprod.csv;

import com.mindprod.hunkio.HunkIO;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import static java.lang.System.err;

/**
 * reshape a CSV file reordering, duplicating or removing fields.
 * <p/>
 * Use: java.exe com.mindprod.CSVReshape somefile.csv   0 3 1 1
 * (0-based column numbers desired in output, if column in out past the end, will be empty)
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 3.5 2010-12-11 - initial version
 * @since 2010-12-11
 */
public final class CSVReshape
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * pack a CSV file, constructor. Just create. There are no methods to call.
     *
     * @param file          CSV file to be packed to remove excess space and quotes.
     * @param separatorChar field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                      tab.
     * @param quoteChar     char to use to enclose fields containing a separator, usually '\"'. Use (char)0 if
     *                      you don't want a quote character.
     * @param commentChar   char to use to introduce comments.  Use (char) 0 if none.  Only one character allowed.
     *
     * @throws java.io.IOException if problems reading/writing file
     */
    @SuppressWarnings( { "WeakerAccess" } )
    public CSVReshape( File file, char separatorChar, char quoteChar, char commentChar, int... colsWanted ) throws IOException
    {
    final String commentChars = ( commentChar == 0 ) ? "" : String.valueOf( commentChar );

    // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
    CSVReader in = new CSVReader( new BufferedReader( new FileReader( file ) ), separatorChar, quoteChar, commentChars, false, true, true );

    File tempFile = HunkIO.createTempFile( "temp", ".tmp", file );

    // writer, quoteLevel, separatorChar, quoteChar, commentChar, trim
    CSVWriter out = new CSVWriter( new PrintWriter( new BufferedWriter( new FileWriter( tempFile ) ) ), 0 /* minimal  */, separatorChar, quoteChar, commentChar, true );

    try
        {
        while ( true )
            {
            // process one line each time through the loop.
            final String[] fields = in.getAllFieldsInLine();
            final int fieldCount;
            final String comment;
            if ( in.wasComment() )
                {
                fieldCount = fields.length - 1; // don't count the comment as a field
                comment = fields[ fieldCount ]; // comment was last, and possibly only field on line.
                }
            else
                {
                fieldCount = fields.length;
                comment = null;
                }

            // output the the fields in desired order.
            for ( int source : colsWanted )
                {
                if ( source >= fieldCount )
                    {
                    out.put( "" );  // request was off past the end.
                    }
                else
                    {
                    out.put( fields[ source ] );  // was actual data field available.
                    }
                }
            // fields copied, now tack on comment, no matter what column it is in.
            if ( comment != null )
                {
                out.nl( comment );
                }
            else
                {
                out.nl();
                }
            } // end while
        }
    catch ( EOFException e )
        {
        in.close();
        out.close();
        file.delete();
        tempFile.renameTo( file );
        }
    }

    // --------------------------- main() method ---------------------------

    /**
     * Simple command line interface to Shape Packs one csv file whose name is on the command line. Must have
     * extension .csv <br> Use java com.mindprod.CSVReshape somefile.csv 0 1 2 3 ...
     * Output replaces input. If you wan the input, make a copy first.
     *
     * @param args name of csv file to remove excess quotes and space, followed by zero-based cols wanted in desired order.
     */
    public static void main(final String[] args )
    {
    if ( args.length < 2 )
        {
        throw new IllegalArgumentException(
                "CSVReshape needs a single filename.csv on the command line followed by 0-based numeric columns desired." );
        }
  final  String filename = args[ 0 ];
    if ( !filename.endsWith( ".csv" ) )
        {
        throw new IllegalArgumentException(
                "Bad Extension\nCSVReshape needs a single filename.csv on the command line followed by 0-based numeric columns desired." );
        }
    final int[] colsWanted = new int[ args.length - 1 ];
    try
        {
        for ( int i = 1; i < args.length; i++ )
            {
            colsWanted[ i - 1 ] = Integer.parseInt( args[ i ] );
            }
        }
    catch ( NumberFormatException e )
        {
        throw new IllegalArgumentException( "CSVReshape needs a single filename.csv on the command line followed by 0-based numeric columns desired." );
        }
    try
        {
        // file,  separatorChar,  quoteChar,  commentChar
        new CSVReshape( new File( filename ), ',', '\"', '#', colsWanted );
        }
    catch ( IOException e )
        {
        err.println( "CSVReshape failed to reshape " + filename );
        err.println( e.getMessage() );
        e.printStackTrace();
        }
    }// end main
    }
