/*
 * @(#)Lines2CSV.java
 *
 * Summary: Converts text file of one field per line to a comma-separated CSV file.
 *
 * Copyright: (c) 2008-2010 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: IntelliJ IDEA IDE.
 *
 * Version History:
 *  1.0 2010-12-14 - initial version
 */
package com.mindprod.csv;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import static java.lang.System.err;

/**
 * Converts text file of one field per line to a comma-separated CSV file.
 * <p/>
 * Use: java.exe com.mindprod.Lines2CSV somefile.txt  3
 * How many lines should be grouped together on one line of the CSV file.
 * Output is somefile.csv.  somefile.txt is left unchanged.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 2010-12-14 - initial version
 * @since 2010-12-14
 */
public final class Lines2CSV
    {
    // -------------------------- PUBLIC INSTANCE  METHODS --------------------------

    /**
     * convert a txt file, constructor. Just create. There are no methods to call.
     *
     * @param file          CSV file to be packed to remove excess space and quotes.
     * @param separatorChar field separator character, usually ',' in North America, ';' in Europe and sometimes '\t' for
     *                      tab for the output file.  It is tab for the input file.
     *                      Note this is a 'char' not a "string".
     * @param quoteChar     character used to quote fields containing awkward chars.
     * @param commentChar   char to use to introduce comments.  Use (char) 0 if none.  Only one character allowed.
     *                      Note this is a 'char' not a "string". For output file. No comment presumed in input file.
     *
     * @throws java.io.IOException if problems reading/writing file
     */
    @SuppressWarnings( { "WeakerAccess" } )
    public Lines2CSV( File file, int grouping, char separatorChar, char quoteChar, char commentChar ) throws IOException
    {
    // O P E N
    final FileReader fr = new FileReader( file );
    final BufferedReader in = new BufferedReader( fr, 24576 /* 24K chars (48K bytes), 75% of allocation is optimal */ );

    // reader, separatorChar, quoteChar, commentChars, hideComments, trimQuoted, allowMultipleLineFields
    String outFilename = file.getAbsolutePath();
    int place = outFilename.lastIndexOf( '.' );
    if ( place < 0 )
        {
        outFilename = outFilename + ".csv";
        }

    else
        {
        outFilename = outFilename.substring( 0, place ) + ".csv";
        }
    // writer, quoteLevel, separatorChar, quoteChar, commentChar, trim
    CSVWriter out = new CSVWriter( new PrintWriter( new BufferedWriter( new FileWriter( outFilename ) ) ), 0 /* minimal */, separatorChar, quoteChar, commentChar, true );

    int count = grouping;

    try
        {
        String thisLine;
        while ( ( thisLine = in.readLine() ) != null )
            {
            out.put( thisLine );
            count--;
            if ( count == 0 )
                {
                out.nl();
                count = grouping;
                }
            }
        }
    catch ( EOFException e )
        {
        }
    finally
        {
        in.close();
        out.close();
        }
    }

    // --------------------------- main() method ---------------------------

    /**
     * Simple command line interface to Lines2CSV. P Converts text file of one field per line to a comma-separated CSV file.
     * java.exe com.mindprod.Lines2CSV somefile.txt  3
     * How many lines should be grouped together on one line of the CSV file.
     * Output is somefile.csv.  somefile.txt is left unchanged.
     * You can use Lines2CSV constructor
     * in your own programs.
     *
     * @param args name of csv file to remove excess quotes and space
     */
    public static void main( String[] args )
    {
    if ( args.length != 2 )
        {
        throw new IllegalArgumentException(
                "Lines2CSV needs a single filename.txt on the command line following by the grouping number." );
        }
    String filename = args[ 0 ];
    int grouping = Integer.parseInt( args[ 1 ] );
    if ( !filename.endsWith( ".txt" ) )
        {
        throw new IllegalArgumentException(
                "Bad Extension\nLines2CSV needs a single filename.txt on the command line following by the grouping number." );
        }
    try
        {
        // file,  separatorChar,  quoteChar,  commentChar
        new Lines2CSV( new File( filename ), grouping, ',', '\"', '#' );
        }
    catch ( IOException e )
        {
        err.println( "Lines2CSV failed to convert " + filename );
        err.println( e.getMessage() );
        e.printStackTrace();
        }
    }// end main
    }
